@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2015 Nullsoft and Contributors
+ * Copyright (C) 1999-2018 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,11 +12,15 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Unicode support by Jim Park -- 08/24/2007
  */
 
 #ifndef __COMPRESSOR_H__
 #define __COMPRESSOR_H__
 
+#include "tchar.h"
+
 #define C_OK 0
 #define C_FINISHED 1
 
@@ -38,9 +42,9 @@ class ICompressor {
     virtual unsigned int GetAvailIn() = 0;
     virtual unsigned int GetAvailOut() = 0;
 
-    virtual const char* GetName() = 0;
+    virtual const TCHAR* GetName() = 0;
 
-    virtual const char* GetErrStr(int err) = 0;
+    virtual const TCHAR* GetErrStr(int err) = 0;
 };
 
 #endif
