@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2015 Nullsoft and Contributors
+ * Copyright (C) 1999-2018 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,6 +12,8 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Unicode support by Jim Park -- 08/13/2007
  */
 
 
@@ -21,7 +23,7 @@
 extern int *cur_langtable;
 
 extern int NSISCALL ui_doinstall(void);
-void NSISCALL update_status_text(int strtab, const char *text2);
+void NSISCALL update_status_text(int strtab, const TCHAR *text2);
 
 extern int ui_dlg_visible;
 extern HWND m_curwnd;
