@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2015 Nullsoft and Contributors
+ * Copyright (C) 1999-2018 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,6 +12,8 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Unicode support by Jim Park -- 08/13/2007
  */
 
 #ifndef ___MAKENSIS_FILEFORM_H___
@@ -38,13 +40,27 @@
     } \
   }
 
+#define DECLARE_PLATFORMITEMWRITER(x) class x##_writer : public writer \
+  { public: \
+    x##_writer(writer_sink *sink) : writer(sink) {} \
+    void write(const x *data, const writer_target_info&ti); \
+    static void write_block(IGrowBuf *pGB, writer_sink *pS, const writer_target_info *pTI = 0) \
+    { \
+      x##_writer writer(pS); \
+      if (!pTI) pTI = &pS->get_target_info(); /* Defaults to TI from sink */ \
+      for (size_t l = pGB->getlen() / sizeof(x), i = 0; i < l; i++) \
+        writer.write(&(((x*)pGB->get())[i]), *pTI); \
+    } \
+  }
+
+
 DECLARE_WRITER(firstheader);
-DECLARE_WRITER(block_header);
-DECLARE_WRITER(header);
+DECLARE_PLATFORMITEMWRITER(block_header);
+DECLARE_PLATFORMITEMWRITER(header); // Platform specific because it writes block_headers
 DECLARE_WRITER(section);
 DECLARE_WRITER(entry);
 DECLARE_WRITER(page);
-DECLARE_WRITER(ctlcolors);
+DECLARE_PLATFORMITEMWRITER(ctlcolors);
 DECLARE_WRITER(LOGFONT);
 
 class lang_table_writer : public writer
